"""Constants for this module."""

EXIT_CODE = "exit_code"
STDOUT = "stdout"
STDERR = "stderr"

ECHO_EXIT_CODE = "echo $?"
DEFAULT_IMAGE = "composio/composio"

ENV_COMPOSIO_DEV_MODE = "COMPOSIO_DEV_MODE"
ENV_COMPOSIO_TOOLSERVER_IMAGE = "COMPOSIO_TOOLSERVER_IMAGE"
