from .click_element import ClickElement
from .get_element_details import GetElementDetails
from .get_page_details import GetPageDetails
from .get_screenshot import GetScreenshot
from .goto_page import GotoPage
from .navigate_history import NavigateHistory
from .refresh_page import RefreshPage
from .scroll_page import ScrollPage
from .type_text import TypeText
